@@ -16,7 +16,9 @@ from .sim.sim_signals import ReadOnlySignal
 from .sim.sim_waveform import SimWaveform
 
 SynSignalRO = ReadOnlySignal
+from .devices.psi_motor import EpicsMotor, EpicsMotorEC
 from .devices.softpositioner import SoftPositioner
+from .interfaces.base_classes.psi_device_base import PSIDeviceBase
 from .utils.bec_device_base import BECDeviceBase
 from .utils.bec_signals import *
 from .utils.dynamic_pseudo import ComputedSignal
