@@ -1,7 +1,7 @@
-from ophyd import EpicsMotor, EpicsSignal, EpicsSignalRO
 from ophyd.quadem import QuadEM
 from ophyd.sim import SynAxis, SynPeriodicSignal, SynSignal
 
 from .epics_motor_ex import EpicsMotorEx
+from .psi_motor import EpicsMotor, EpicsMotorEC
 from .sls_devices import SLSInfo, SLSOperatorMessages
 from .SpmBase import SpmBase
