@@ -3,6 +3,7 @@ Module for custom BEC signals, that wrap around ophyd.Signal.
 These signals emit BECMessage objects, which comply with the BEC message system.
 """
 
+import time
 from typing import Any, Callable, Literal, Type
 
 import numpy as np
@@ -10,6 +11,7 @@ from bec_lib import messages
 from bec_lib.logger import bec_logger
 from ophyd import DeviceStatus, Kind, Signal
 from pydantic import BaseModel, Field, ValidationError
+from typeguard import typechecked
 
 logger = bec_logger.logger
 # pylint: disable=arguments-differ
@@ -60,6 +62,14 @@ class SignalInfo(BaseModel):
         default=None,
         description="Metadata for the signal, which can include additional information about the signal's properties.",
     )
+    acquisition_group: Literal["baseline", "monitored"] | str | None = Field(
+        default=None,
+        description="""Specifies the acquisition group of the signal.
+        It can be in sync with 'baseline' or 'monitored' groups mapping readoutPriority.
+        Or mapped to a custom tag that allows grouping signals for acquisition and plotting.
+        If None, the signal does not belong to any specific acquisition group.
+        """,
+    )
 
 
 class BECMessageSignal(Signal):
@@ -79,6 +89,7 @@ class BECMessageSignal(Signal):
         ndim: Literal[0, 1, 2] | None = None,
         scope: Literal["scan", "continuous"] = "scan",
         role: Literal["main", "preview", "diagnostic", "file event", "progress"] = "main",
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
         enabled: bool = True,
         signals: (
             Callable[[], list[str]]
@@ -116,8 +127,9 @@ class BECMessageSignal(Signal):
         self.scope = scope
         self.role = role
         self.enabled = enabled
+        self.acquisition_group = acquisition_group
         self.signals = self._unify_signals(signals)
-        self.signal_metadata = signal_metadata
+        self.signal_metadata = signal_metadata or {}
         self._bec_message_type = bec_message_type
 
     def _unify_signals(
@@ -135,26 +147,34 @@ class BECMessageSignal(Signal):
         if isinstance(signals, Callable):
             signals = signals()
         if signals is None:
-            return [(self.attr_name or self.name, Kind.hinted.value)]
+            return [(self.name, Kind.hinted.value)]  # Default to signal name with hinted kind
         if isinstance(signals, str):
-            return [(signals, Kind.hinted.value)]
-        if not isinstance(signals, list):
-            raise ValueError(
-                f"Signals must be a list of tuples or strings, got {type(signals).__name__}."
-            )
-        out = []
-        for signal in signals:
-            if isinstance(signal, str):
-                out.append((signal, Kind.normal.value))
-            elif isinstance(signal, tuple) and len(signal) == 2:
-                if isinstance(signal[1], Kind):
-                    out.append((signal[0], signal[1].value))
-                else:
-                    out.append((signal[0], signal[1]))
-            else:
+            out = [(signals, Kind.hinted.value)]
+        else:
+            if not isinstance(signals, list):
                 raise ValueError(
-                    f"Invalid signal format: {signal}. Expected a tuple of (name, kind) or a string."
+                    f"Signals must be a list of tuples or strings, got {type(signals).__name__}."
                 )
+            out = []
+            for signal in signals:
+                if isinstance(signal, str):
+                    kind = Kind.hinted.value if len(signals) == 1 else Kind.normal.value
+                    out.append((signal, kind))
+                elif isinstance(signal, tuple) and len(signal) == 2:
+                    if isinstance(signal[1], Kind):
+                        out.append((signal[0], signal[1].value))
+                    else:
+                        out.append((signal[0], signal[1]))
+                else:
+                    raise ValueError(
+                        f"Invalid signal format: {signal}. Expected a tuple of (name, kind) or a string."
+                    )
+        if len(out) == 1 and out[0][0] != self.name:
+            signal_name, signal_kind = out[0]
+            logger.warning(
+                f"Signal {self.name} of class {self.__class__.__name__} has only one sub-signal. Signal name {signal_name} will be renamed to {self.name}."
+            )
+            out = [(self.name, signal_kind)]
         return out
 
     def describe(self):
@@ -169,6 +189,7 @@ class BECMessageSignal(Signal):
             enabled=self.enabled,
             signals=self.signals,
             signal_metadata=self.signal_metadata,
+            acquisition_group=self.acquisition_group,
         ).model_dump()
         return out
 
@@ -303,7 +324,7 @@ class ProgressSignal(BECMessageSignal):
 
         try:
             msg = messages.ProgressMessage(
-                value=value, max_value=max_value, done=done, metadata=metadata
+                value=value, max_value=max_value, done=done, metadata=metadata or {}
             )
         except ValidationError as exc:
             raise ValueError(f"Error setting signal {self.name}: {exc}") from exc
@@ -441,7 +462,7 @@ class FileEventSignal(BECMessageSignal):
                 device_name=device_name,
                 file_type=file_type,
                 hinted_h5_entries=hinted_h5_entries,
-                metadata=metadata,
+                metadata=metadata or {},
             )
         except ValidationError as exc:
             raise ValueError(f"Error setting signal {self.name}: {exc}") from exc
@@ -519,8 +540,6 @@ class PreviewSignal(BECMessageSignal):
             transpose (bool): Whether to transpose the data for visualization.
             value (DeviceMonitorMessage | dict | None): The initial value of the signal. Defaults to None.
         """
-        self.num_rotation_90 = num_rotation_90
-        self.transpose = transpose
         kwargs.pop("kind", None)
         super().__init__(
             name=name,
@@ -535,6 +554,24 @@ class PreviewSignal(BECMessageSignal):
             **kwargs,
         )
 
+    @property
+    def num_rotation_90(self) -> Literal[0, 1, 2, 3]:
+        """Get the number of 90 degree counter-clockwise rotations applied to the data."""
+        return self.signal_metadata["num_rotation_90"]
+
+    @num_rotation_90.setter
+    def num_rotation_90(self, value: Literal[0, 1, 2, 3]) -> None:
+        self.signal_metadata["num_rotation_90"] = value
+
+    @property
+    def transpose(self) -> bool:
+        """Get whether the data is transposed."""
+        return self.signal_metadata["transpose"]
+
+    @transpose.setter
+    def transpose(self, value: bool) -> None:
+        self.signal_metadata["transpose"] = value
+
     def _process_data(self, value: np.ndarray) -> np.ndarray:
         if self.ndim == 1:
             return value
@@ -587,7 +624,7 @@ class PreviewSignal(BECMessageSignal):
             value = self._process_data(value)
             try:
                 msg = messages.DevicePreviewMessage(
-                    data=value, device=device_name, signal=signal_name, metadata=metadata
+                    data=value, device=device_name, signal=signal_name, metadata=metadata or {}
                 )
             except ValidationError as exc:
                 raise ValueError(f"Error setting signal {self.name}: {exc}") from exc
@@ -618,7 +655,7 @@ class PreviewSignal(BECMessageSignal):
 
 
 class DynamicSignal(BECMessageSignal):
-    """Signal to emit dynamic device data."""
+    """Signal group to emit dynamic device signal data."""
 
     strict_signal_validation = False  # Disable strict signal validation
 
@@ -626,19 +663,31 @@ class DynamicSignal(BECMessageSignal):
         self,
         *,
         name: str,
-        signals: list[str] | Callable[[], list[str]] | None = None,
+        signals: list[str] | Callable[[], list[str]] | str | None = None,
         value: messages.DeviceMessage | dict | None = None,
-        async_update: dict | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
         **kwargs,
     ):
         """
         Create a new DynamicSignal object.
 
         Args:
-            name (str): The name of the signal.
-            signal_names (list[str] | Callable): The names of all signals. Can be a list or a callable.
+            name (str): The name of the signal group.
+            signal_names (list[str] | Callable): Names of all signals. Can be a list or a callable.
             value (DeviceMessage | dict | None): The initial value of the signal. Defaults to None.
-            async_update (dict | None): Additional metadata for asynchronous updates. Defaults to None.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal group.
+            async_update (dict | None): Additional metadata for asynchronous updates.
+                                        There are three relevant keys "type", "max_shape" and "index".
+                                        "type" (str) : Can be one of "add", "add_slice" or "replace". This defines how the new data is added to the existing dataset.
+                                                "add" : Appends data to the existing dataset. The data is always appended to the first axis.
+                                                "add_slice" : Appends data to the existing dataset, but allows specifying a slice.
+                                                              The slice is defined by the "index" key.
+                                                "replace" : Replaces the existing dataset with the new data.
+                                        "max_shape" (list[int | None]): Required for type 'add' and 'add_slice'. It defines where the data is added. For a 1D dataset,
+                                                                        it should be [None]. For a 1D dataset with 3000 elements, it should be [None, 3000].
+                                                                        For a 2D dataset with 3000x3000 elements, it should be [None, 3000, 3000].
+                                        "index" (int): Only required for type 'add_slice'. It defines the index where the data is added.
         """
         self.async_update = async_update
 
@@ -653,15 +702,18 @@ class DynamicSignal(BECMessageSignal):
             signals=signals,
             value=value,
             bec_message_type=kwargs.pop("bec_message_type", messages.DeviceMessage),
+            acquisition_group=acquisition_group,
             **kwargs,
         )
 
+    @typechecked
     def put(
         self,
         value: messages.DeviceMessage | dict[str, dict[Literal["value", "timestamp"], Any]],
         *,
         metadata: dict | None = None,
-        async_update: dict | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
         **kwargs,
     ) -> None:
         """
@@ -674,37 +726,59 @@ class DynamicSignal(BECMessageSignal):
         Args:
             value (dict | DeviceMessage): The dynamic device data.
             metadata (dict | None): Additional metadata.
+            async_update (dict[Literal["type", "max_shape", "index"], Any] | None): Additional metadata for asynchronous updates.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal group.
         """
         if isinstance(value, messages.DeviceMessage):
-            if metadata is not None or async_update is not None:
+            if metadata is not None or async_update is not None or acquisition_group is not None:
                 logger.warning(
-                    "Ignoring metadata and async_update arguments when value is a DeviceMessage."
+                    "Ignoring metadata, async_update and acquisition_group arguments when value is a DeviceMessage."
                 )
             self._check_signals(value)
+            self._check_async_update(value)
             return super().put(value, **kwargs)
         try:
             metadata = metadata or {}
-            if "async_update" not in metadata:
-                if async_update is not None:
-                    metadata["async_update"] = async_update
-                elif self.async_update is not None:
-                    metadata["async_update"] = self.async_update
+            if async_update is not None:
+                metadata["async_update"] = async_update
+            elif self.async_update is not None:
+                metadata["async_update"] = self.async_update
+            if acquisition_group is not None:
+                metadata["acquisition_group"] = acquisition_group
+            elif self.acquisition_group is not None:
+                metadata["acquisition_group"] = self.acquisition_group
 
             msg = messages.DeviceMessage(signals=value, metadata=metadata)
         except ValidationError as exc:
             raise ValueError(f"Error setting signal {self.name}: {exc}") from exc
         self._check_signals(msg)
+        self._check_async_update(msg)
         return super().put(msg, **kwargs)
 
+    def _check_async_update(self, msg: messages.DeviceMessage) -> None:
+        """Check if async_update metadata is present."""
+        if "async_update" not in msg.metadata:
+            raise ValueError(
+                f"Async update must be provided for signal {self.name} of class {self.__class__.__name__}."
+            )
+        if not isinstance(msg.metadata["async_update"], dict):
+            raise ValueError(
+                f"Async update metadata must be a dict for signal {self.name} of class {self.__class__.__name__}."
+            )
+
+        # Validate async_update using DeviceAsyncUpdate model
+        messages.DeviceAsyncUpdate(**msg.metadata["async_update"])
+
     def _check_signals(self, msg: messages.DeviceMessage) -> None:
-        """Check if all signals are valid."""
+        """Check if all signals are valid, and if relevant metadata is also present."""
         if len(self.signals) == 1:
             if self.name not in msg.signals:
                 raise ValueError(
                     f"Signal {self.name} not found in message {list(msg.signals.keys())}"
                 )
             return
-        available_signals = [f"{self.name}_{name}" for name, _ in self.signals]
+        self._normalize_signals(msg)
+        available_signals = [f"{self.name}_{signal_name}" for signal_name, _ in self.signals]
         if self.strict_signal_validation:
             if set(msg.signals.keys()) != set(available_signals):
                 raise ValueError(
@@ -717,13 +791,43 @@ class DynamicSignal(BECMessageSignal):
                 raise ValueError(
                     f"Invalid signal name in message {list(msg.signals.keys())} for signals {available_signals}"
                 )
+        # Check if async_update metadata is present
+        if "async_update" not in msg.metadata:
+            raise ValueError(
+                f"Async update must be provided for signal {self.name} of class {self.__class__.__name__}."
+            )
+        # Add here validation for async update
+        # TODO #629 Issue in BEC: Validate async_update --> bec_lib
+
+    def _normalize_signals(self, msg: messages.DeviceMessage) -> None:
+        """
+        Normalize signal names in the message to include the group name as prefix.
+        For a device 'samx' and signal component 'mysignal' with sub-signals 'a', 'b', 'c',
+        the expected signal names are either
+            'samx_mysignal_a', 'samx_mysignal_b', 'samx_mysignal_c'
+        or just
+            'a', 'b', 'c'
+        This method normalizes the latter case to the former.
+
+        Args:
+            msg (DeviceMessage): The device message to normalize.
+        """
+        prefix = f"{self.name}_"
+        normalized_signals = {}
+        for signal_name, data in msg.signals.items():
+            if signal_name.startswith(prefix):
+                normalized_signals[signal_name] = data
+            else:
+                normalized_signals[f"{prefix}{signal_name}"] = data
+        msg.signals = normalized_signals
 
     def set(
         self,
         value: messages.DeviceMessage | dict[str, dict[Literal["value"], Any]],
         *,
         metadata: dict | None = None,
-        async_update: dict | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
         **kwargs,
     ) -> DeviceStatus:
         """
@@ -737,14 +841,20 @@ class DynamicSignal(BECMessageSignal):
             value (dict | DeviceMessage)                : The dynamic device data.
             metadata (dict | None)                      : Additional metadata.
         """
-        self.put(value, metadata=metadata, async_update=async_update, **kwargs)
+        self.put(
+            value,
+            metadata=metadata,
+            async_update=async_update,
+            acquisition_group=acquisition_group,
+            **kwargs,
+        )
         status = DeviceStatus(device=self)
         status.set_finished()
         return status
 
 
-class AsyncSignal(DynamicSignal):
-    """Signal to emit asynchronous data."""
+class AsyncMultiSignal(DynamicSignal):
+    """Async Signal group to emit asynchronous data from multiple signals."""
 
     strict_signal_validation = True
 
@@ -754,19 +864,33 @@ class AsyncSignal(DynamicSignal):
         name: str,
         ndim: Literal[0, 1, 2],
         max_size: int,
+        signals: list[str] | Callable[[], list[str]],
         value: messages.DeviceMessage | dict | None = None,
-        async_update: dict | None = None,
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
         **kwargs,
     ):
         """
         Create a new AsyncSignal object.
 
         Args:
-            name (str): The name of the signal.
-            ndim (Literal[0, 1, 2]): The number of dimensions of the signal(s).
-            max_size (int): The maximum size of the signal buffer.
+            name (str): The name of the signal group.
+            ndim (Literal[0, 1, 2]): The number of dimensions of the signals.
+            max_size (int): The maximum size of the signal buffer. For ndim=2, this should be kept small to avoid large memory usage.
+            signals (list[str] | Callable[[], list[str]]): The names of all sub-signals. Names will be prefixed with the group name.
             value (AsyncMessage | dict | None): The initial value of the signal. Defaults to None.
-            async_update (dict | None): Additional metadata for asynchronous updates. Defaults to None.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal group.
+            async_update (dict | None): Additional metadata for asynchronous updates.
+                                        There are three relevant keys "type", "max_shape" and "index".
+                                        "type" (str) : Can be one of "add", "add_slice" or "replace". This defines how the new data is added to the existing dataset.
+                                                "add" : Appends data to the existing dataset. The data is always appended to the first axis.
+                                                "add_slice" : Appends data to the existing dataset, but allows specifying a slice.
+                                                              The slice is defined by the "index" key.
+                                                "replace" : Replaces the existing dataset with the new data.
+                                        "max_shape" (list[int | None]): Required for type 'add' and 'add_slice'. It defines where the data is added. For a 1D dataset,
+                                                                        it should be [None]. For a 1D dataset with 3000 elements, it should be [None, 3000].
+                                                                        For a 2D dataset with 3000x3000 elements, it should be [None, 3000, 3000].
+                                        "index" (int): Only required for type 'add_slice'. It defines the index where the data is added.
         """
         kwargs.pop("kind", None)  # Ignore kind if specified
         super().__init__(
@@ -780,5 +904,124 @@ class AsyncSignal(DynamicSignal):
             bec_message_type=messages.DeviceMessage,
             async_update=async_update,
             signal_metadata={"max_size": max_size},
+            acquisition_group=acquisition_group,
+            signals=signals,
             **kwargs,
         )
+
+
+class AsyncSignal(DynamicSignal):
+    """Device Signal to emit data asynchronously."""
+
+    strict_signal_validation = True
+
+    def __init__(
+        self,
+        *,
+        name: str,
+        ndim: Literal[0, 1, 2],
+        max_size: int,
+        value: messages.DeviceMessage | dict | None = None,
+        acquisition_group: Literal["baseline", "monitored"] | str | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        **kwargs,
+    ):
+        """
+        Create a new AsyncSignal object.
+
+        Args:
+            name (str): The name of the signal.
+            ndim (Literal[0, 1, 2]): The number of dimensions of the signals.
+            max_size (int): The maximum size of the signal buffer. For ndim=2, this should be kept small to avoid large memory usage.
+            value (AsyncMessage | dict | None): The initial value of the signal. Defaults to None.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal group.
+            async_update (dict | None): Additional metadata for asynchronous updates.
+                                        There are three relevant keys "type", "max_shape" and "index".
+                                        "type" (str) : Can be one of "add", "add_slice" or "replace". This defines how the new data is added to the existing dataset.
+                                                "add" : Appends data to the existing dataset. The data is always appended to the first axis.
+                                                "add_slice" : Appends data to the existing dataset, but allows specifying a slice.
+                                                              The slice is defined by the "index" key.
+                                                "replace" : Replaces the existing dataset with the new data.
+                                        "max_shape" (list[int | None]): Required for type 'add' and 'add_slice'. It defines where the data is added. For a 1D dataset,
+                                                                        it should be [None]. For a 1D dataset with 3000 elements, it should be [None, 3000].
+                                                                        For a 2D dataset with 3000x3000 elements, it should be [None, 3000, 3000].
+                                        "index" (int): Only required for type 'add_slice'. It defines the index where the data is added.
+        """
+        kwargs.pop("kind", None)  # Ignore kind if specified
+        super().__init__(
+            name=name,
+            data_type="raw",
+            saved=True,
+            ndim=ndim,
+            scope="scan",
+            role="main",
+            value=value,
+            bec_message_type=messages.DeviceMessage,
+            async_update=async_update,
+            signal_metadata={"max_size": max_size},
+            acquisition_group=acquisition_group,
+            signals=None,
+            **kwargs,
+        )
+
+    def put(
+        self,
+        value: Any,
+        timestamp: float | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        acquisition_group: str | None = None,
+        **kwargs,
+    ) -> None:
+        """
+        Put method for AsyncSignal.
+
+        Args:
+            value (Any): The value to put.
+            timestamp (float | None): The timestamp of the value. If None, the current time is used.
+            async_update (dict[Literal["type", "max_shape", "index"], Any] | None): Additional metadata for asynchronous updates. Please refer to the class docstring for details.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal.
+        """
+        timestamp = timestamp or time.time()
+        super().put(
+            value={self.name: {"value": value, "timestamp": timestamp}},
+            async_update=async_update,
+            acquisition_group=acquisition_group,
+            **kwargs,
+        )
+
+    def set(
+        self,
+        value: Any,
+        timestamp: float | None = None,
+        async_update: dict[Literal["type", "max_shape", "index"], Any] | None = None,
+        acquisition_group: str | None = None,
+        **kwargs,
+    ) -> DeviceStatus:
+        """
+        Set method for AsyncSignal.
+
+        Args:
+            value (Any): The value to put.
+            timestamp (float | None): The timestamp of the value. If None, the current time is used.
+            async_update (dict[Literal["type", "max_shape", "index"], Any] | None): Additional metadata for asynchronous updates. Please refer to the class docstring for details.
+            acquisition_group (Literal["baseline", "monitored"] | str | None): The acquisition group of the signal.
+        """
+        self.put(
+            value=value,
+            timestamp=timestamp,
+            async_update=async_update,
+            acquisition_group=acquisition_group,
+            **kwargs,
+        )
+        status = DeviceStatus(device=self)
+        status.set_finished()
+        return status
+
+    @property
+    def max_size(self) -> int:
+        """Get the maximum size of the signal buffer."""
+        return self.signal_metadata["max_size"]
+
+    @max_size.setter
+    def max_size(self, value: int) -> None:
+        self.signal_metadata["max_size"] = value
