@@ -1,16 +1,29 @@
 import threading
 import time
+from functools import partial
 from unittest import mock
 
 import numpy as np
 import ophyd
 import pytest
 from bec_lib import messages
+from ophyd import Component as Cpt
 from ophyd import Device, EpicsSignalRO, Signal
 from ophyd.status import WaitTimeoutError
+from typeguard import TypeCheckError
 
-from ophyd_devices.tests.utils import MockPV, patch_dual_pvs
+from ophyd_devices import (
+    AndStatus,
+    DeviceStatus,
+    MoveStatus,
+    Status,
+    StatusBase,
+    SubscriptionStatus,
+)
+from ophyd_devices.tests.utils import MockPV
 from ophyd_devices.utils.bec_signals import (
+    AsyncMultiSignal,
+    AsyncSignal,
     BECMessageSignal,
     DynamicSignal,
     FileEventSignal,
@@ -71,8 +84,8 @@ def test_utils_file_handler_has_full_path(file_handler):
 
 def test_utils_task_status(device):
     """Test TaskStatus creation"""
-    status = TaskStatus(device=device)
-    assert status.device.name == "device"
+    status = TaskStatus(device)
+    assert status.obj.name == "device"
     assert status.state == "not_started"
     assert status.task_id == status._task_id
     status.state = "running"
@@ -247,7 +260,8 @@ def test_utils_bec_message_signal():
                 "enabled": True,
                 "rpc_access": False,
                 "signals": [("bec_message_signal", 5)],
-                "signal_metadata": None,
+                "signal_metadata": {},
+                "acquisition_group": None,
             },
         }
     }
@@ -270,6 +284,41 @@ def test_utils_bec_message_signal():
         signal.put({"wrong_key": "wrong_value"})
 
 
+@pytest.mark.parametrize(
+    "input_msg, output_msg",
+    [
+        (
+            messages.DeviceMessage(
+                signals={"sig1": {"value": 1}, "sig2": {"value": 2}}, metadata={"info": "test"}
+            ),
+            messages.DeviceMessage(
+                signals={"device_data_sig1": {"value": 1}, "device_data_sig2": {"value": 2}},
+                metadata={"info": "test"},
+            ),
+        ),
+        (
+            messages.DeviceMessage(
+                signals={"device_data_sig1": {"value": 1}, "device_data_sig2": {"value": 2}},
+                metadata={"info": "test"},
+            ),
+            messages.DeviceMessage(
+                signals={"device_data_sig1": {"value": 1}, "device_data_sig2": {"value": 2}},
+                metadata={"info": "test"},
+            ),
+        ),
+    ],
+)
+def test_utils_signal_normalization(input_msg, output_msg):
+    """Test signal normalization utility in BECMessageSignal"""
+
+    class DeviceWithSignal(Device):
+        data = Cpt(AsyncMultiSignal, name="data", signals=["sig1", "sig2"], ndim=0, max_size=1000)
+
+    dev = DeviceWithSignal(name="device")
+    dev.data._normalize_signals(input_msg)
+    assert input_msg == output_msg
+
+
 def test_utils_dynamic_signal():
     """Test DynamicSignal"""
     dev = Device(name="device")
@@ -293,29 +342,130 @@ def test_utils_dynamic_signal():
                 "enabled": True,
                 "rpc_access": False,
                 "signals": [("sig1", 1), ("sig2", 1)],
-                "signal_metadata": None,
+                "signal_metadata": {},
+                "acquisition_group": None,
             },
         }
     }
 
     # Put works with Message
     msg_dict = {"dynamic_signal_sig1": {"value": 1}, "dynamic_signal_sig2": {"value": 2}}
-    msg = messages.DeviceMessage(signals=msg_dict)
+    with pytest.raises(ValueError):
+        # Missing metadata
+        signal.put(messages.DeviceMessage(signals=msg_dict))
+    metadata = {"async_update": {"type": "add", "max_shape": [None, 1000]}}
+    msg = messages.DeviceMessage(signals=msg_dict, metadata=metadata)
     signal.put(msg)
     reading = signal.read()
     assert reading[signal.name]["value"] == msg
     # Set works with dict
-    status = signal.set(msg_dict)
+    status = signal.set(msg_dict, metadata=metadata)
     assert status.done is True
     reading = signal.read()
     assert reading[signal.name]["value"] == msg
     # Put fails with wrong type
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeCheckError):
         signal.put("wrong_type")
     # Put fails with wrong dict
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeCheckError):
         signal.put({"wrong_key": "wrong_value"})
 
+    # Set with acquisition group
+    signal.put(msg, acquisition_group="fly-scan")
+    reading = signal.read()
+    msg.metadata["acquisition_group"] = "fly-scan"
+    assert reading[signal.name]["value"] == msg
+
+
+def test_utils_dynamic_signal_with_defaults():
+    """
+    Test DynamicSignal with async_update and acquisition group defaults. If only
+    one sub-signal is provided for the dynamic signal, the name of the sub-signal
+    will be used in the signals dict and a warning will be issued that the sub-signal
+    name is being ignored.
+    """
+    dev = Device(name="device")
+    create_signal = partial(
+        DynamicSignal,
+        name="dynamic_signal",
+        parent=dev,
+        ndim=1,
+        value=None,
+        async_update={"type": "add", "max_shape": [None, 1000]},
+        acquisition_group="fly-scanning",
+    )
+    signal = create_signal(signals=["sig1", "sig2"])
+    val = np.random.random(1000)
+    msg_dict = {"dynamic_signal_sig1": {"value": val}}
+    signal.put(msg_dict)
+    reading = signal.read()
+    reading_value = reading[signal.name]["value"].model_dump(exclude={"timestamp"})
+    assert reading_value["signals"] == msg_dict
+    assert reading_value["metadata"]["async_update"] == {"type": "add", "max_shape": [None, 1000]}
+    assert reading_value["metadata"]["acquisition_group"] == "fly-scanning"
+
+    signal.put(msg_dict, acquisition_group="different-group")
+    reading = signal.read()
+    assert reading[signal.name]["value"].metadata["acquisition_group"] == "different-group"
+
+    # Test init variations for single signal
+    for signal in [["sig1"], "sig1", None]:
+        signal = create_signal(signals=signal)
+        assert signal.signals == [(signal.name, ophyd.Kind.hinted.value)]
+
+
+def test_utils_async_multi_signal():
+    """Test AsyncMultiSignal, which is a DynamicSignal with strict signal validation."""
+    device = Device(name="device")
+    signal = AsyncMultiSignal(
+        name="async_multi_signal",
+        ndim=1,
+        max_size=1000,
+        signals=["sig1", "sig2"],
+        async_update={"type": "add", "max_shape": [None, 1000]},
+        parent=device,
+    )
+    val = np.random.random(1000)
+    msg_dict = {"async_multi_signal_sig1": {"value": val}}
+    with pytest.raises(ValueError):
+        # Missing signal
+        signal.put(msg_dict)
+    msg_dict = {
+        "async_multi_signal_sig1": {"value": val},
+        "async_multi_signal_sig2": {"value": val},
+    }
+    signal.put(msg_dict)
+    reading = signal.read()
+    reading_value = reading[signal.name]["value"].model_dump(exclude={"timestamp"})
+    assert reading_value["signals"] == msg_dict
+    assert reading_value["metadata"]["async_update"] == {"type": "add", "max_shape": [None, 1000]}
+
+
+def test_utils_async_signal():
+    device = Device(name="device")
+    signal = AsyncSignal(
+        name="async_signal",
+        ndim=1,
+        max_size=1000,
+        async_update={"type": "add", "max_shape": [None, 200]},
+        parent=device,
+    )
+    val = np.random.random(1000)
+    signal.put(
+        val,
+        async_update={"type": "add_slice", "max_shape": [None, 1000], "index": 1},
+        acquisition_group="scan",
+    )
+    reading = signal.read()
+    reading_value = reading[signal.name]["value"].model_dump(exclude={"timestamp"})
+    assert np.array_equal(reading_value["signals"][signal.name]["value"], val)
+    assert reading_value["metadata"]["async_update"] == {
+        "type": "add_slice",
+        "max_shape": [None, 1000],
+        "index": 1,
+    }
+    assert reading_value["metadata"]["acquisition_group"] == "scan"
+
 
 def test_utils_file_event_signal():
     """Test FileEventSignal"""
@@ -339,7 +489,8 @@ def test_utils_file_event_signal():
                 "enabled": True,
                 "rpc_access": False,
                 "signals": [("file_event_signal", 5)],
-                "signal_metadata": None,
+                "signal_metadata": {},
+                "acquisition_group": None,
             },
         }
     }
@@ -391,6 +542,7 @@ def test_utils_preview_1d_signal():
                 "rpc_access": False,
                 "signals": [("preview_1d_signal", 5)],
                 "signal_metadata": {"num_rotation_90": 0, "transpose": False},
+                "acquisition_group": None,
             },
         }
     }
@@ -453,6 +605,7 @@ def test_utils_preview_2d_signal():
                 "rpc_access": False,
                 "signals": [("preview_2d_signal", 5)],
                 "signal_metadata": {"num_rotation_90": 0, "transpose": False},
+                "acquisition_group": None,
             },
         }
     }
@@ -517,7 +670,8 @@ def test_utils_progress_signal():
                 "enabled": True,
                 "rpc_access": False,
                 "signals": [("progress_signal", 5)],
-                "signal_metadata": None,
+                "signal_metadata": {},
+                "acquisition_group": None,
             },
         }
     }
@@ -545,18 +699,19 @@ def test_utils_progress_signal():
 
 
 def test_utils_compare_status_number():
-    """Test CompareStatus"""
+    """Test CompareStatus with different operations."""
     sig = Signal(name="test_signal", value=0)
-    status = CompareStatus(signal=sig, value=5, operation="==")
+    status = CompareStatus(signal=sig, value=5, operation_success="==")
     assert status.done is False
     sig.put(1)
     assert status.done is False
     sig.put(5)
+    status.wait(timeout=5)
     assert status.done is True
 
     sig.put(5)
     # Test with different operations
-    status = CompareStatus(signal=sig, value=5, operation="!=")
+    status = CompareStatus(signal=sig, value=5, operation_success="!=")
     assert status.done is False
     sig.put(5)
     assert status.done is False
@@ -566,7 +721,7 @@ def test_utils_compare_status_number():
     assert status.exception() is None
 
     sig.put(0)
-    status = CompareStatus(signal=sig, value=5, operation=">")
+    status = CompareStatus(signal=sig, value=5, operation_success=">")
     assert status.done is False
     sig.put(5)
     assert status.done is False
@@ -575,11 +730,44 @@ def test_utils_compare_status_number():
     assert status.success is True
     assert status.exception() is None
 
+    # Should raise
+    sig.put(0)
+    status = CompareStatus(signal=sig, value=5, operation_success="==", failure_value=[10])
+    with pytest.raises(ValueError):
+        sig.put(10)
+        status.wait()
+    assert status.done is True
+    assert status.success is False
+    assert isinstance(status.exception(), ValueError)
+
+    # failure_operation
+    sig.put(0)
+    status = CompareStatus(
+        signal=sig, value=5, operation_success="==", failure_value=10, operation_failure=">"
+    )
+    sig.put(10)
+    assert status.done is False
+    assert status.success is False
+    sig.put(11)
+    with pytest.raises(ValueError):
+        status.wait()
+    assert status.done is True
+    assert status.success is False
+
+    # raise if array is returned
+    sig.put(0)
+    status = CompareStatus(signal=sig, value=5, operation_success="==")
+    with pytest.raises(ValueError):
+        sig.put([1, 2, 3])
+        status.wait(timeout=2)
+    assert status.done is True
+    assert status.success is False
+
 
 def test_compare_status_string():
     """Test CompareStatus with string values"""
     sig = Signal(name="test_signal", value="test")
-    status = CompareStatus(signal=sig, value="test", operation="==")
+    status = CompareStatus(signal=sig, value="test", operation_success="==")
     assert status.done is False
     sig.put("test1")
     assert status.done is False
@@ -588,7 +776,7 @@ def test_compare_status_string():
 
     sig.put("test")
     # Test with different operations
-    status = CompareStatus(signal=sig, value="test", operation="!=")
+    status = CompareStatus(signal=sig, value="test", operation_success="!=")
     assert status.done is False
     sig.put("test")
     assert status.done is False
@@ -597,12 +785,6 @@ def test_compare_status_string():
     assert status.success is True
     assert status.exception() is None
 
-    # Test with greater than operation
-    # Raises ValueError for strings
-    sig.put("a")
-    with pytest.raises(ValueError):
-        status = CompareStatus(signal=sig, value="b", operation=">")
-
 
 def test_transition_status():
     """Test TransitionStatus"""
@@ -622,9 +804,9 @@ def test_transition_status():
     assert status.success is True
     assert status.exception() is None
 
-    # Test strict=True, ra
+    # Test strict=True, failure_states
     sig.put(1)
-    status = TransitionStatus(signal=sig, transitions=[1, 2, 3], strict=True, raise_states=[4])
+    status = TransitionStatus(signal=sig, transitions=[1, 2, 3], strict=True, failure_states=[4])
     assert status.done is False
     sig.put(4)
     with pytest.raises(ValueError):
@@ -708,7 +890,7 @@ def test_compare_status_with_mock_pv(mock_epics_signal_ro):
     """Test CompareStatus with EpicsSignalRO, this tests callbacks on EpicsSignals"""
 
     signal = mock_epics_signal_ro
-    status = CompareStatus(signal=signal, value=5, operation="==")
+    status = CompareStatus(signal=signal, value=5, operation_success="==")
     assert status.done is False
     signal._read_pv.mock_data = 1
     assert status.done is False
@@ -718,6 +900,19 @@ def test_compare_status_with_mock_pv(mock_epics_signal_ro):
     assert status.success is True
 
 
+def test_compare_status_raises_on_failed_comparison(mock_epics_signal_ro):
+    """Test CompareStatus raises on failed comparison with EpicsSignalRO"""
+
+    signal = mock_epics_signal_ro
+    status = CompareStatus(
+        signal=signal, value=5, operation_success="==", failure_value=[np.array([10])]
+    )
+    assert status.done is False
+    signal._read_pv.mock_data = 1
+    with pytest.raises(Exception):
+        status.wait(timeout=5)
+
+
 @pytest.mark.parametrize(
     "transitions, expected_done, expected_success",
     [
@@ -755,3 +950,74 @@ def test_transition_status_with_mock_pv(
             status.wait(timeout=1)
         assert status.done is False
         assert status.success is False
+
+
+def test_patched_status_objects():
+    """Test the patched Status objects in ophyd_devices that improve error handling."""
+
+    # StatusBase & AndStatus
+    st = StatusBase()
+    st2 = StatusBase()
+    and_st = st & st2
+    assert st in and_st
+    assert isinstance(and_st, AndStatus)
+    st.set_exception(ValueError("test error"))
+    with pytest.raises(ValueError):
+        and_st.wait(timeout=10)
+
+    # DeviceStatus & Status
+    dev = Device(name="device")
+    dev_status = DeviceStatus(device=dev)
+
+    st = Status()
+    and_st = st & dev_status
+    assert dev_status.device == dev
+    dev_status.set_exception(RuntimeError("device error"))
+    with pytest.raises(RuntimeError):
+        and_st.wait(timeout=10)
+
+    # Combine DeviceStatus with StatusBase and form AndStatus
+    st = StatusBase(obj=dev)
+    assert st.obj == dev
+    dev_st = DeviceStatus(device=dev)
+    combined_st = st & dev_st
+    st.set_finished()
+    dev_st.set_exception(RuntimeError("combined error"))
+    with pytest.raises(RuntimeError):
+        combined_st.wait(timeout=10)
+
+    # SubscriptionStatus
+    sig = Signal(name="test_signal", value=0)
+
+    def _cb(*args, **kwargs):
+        pass
+
+    sub_st = SubscriptionStatus(sig, callback=_cb)
+    sub_st.set_exception(ValueError("subscription error"))
+    with pytest.raises(ValueError):
+        sub_st.wait(timeout=10)
+    assert sub_st.done is True
+    assert sub_st.success is False
+
+    # MoveStatus, here the default for call_stop_on_failure is True
+    class Positioner(Device):
+        SUB_READBACK = "readback"
+        setpoint = Signal(name="setpoint", value=0)
+        readback = Signal(name="readback", value=0)
+
+        @property
+        def position(self):
+            return self.readback.get()
+
+        def stop(self):
+            pass
+
+    pos = Positioner(name="positioner")
+    move_st = MoveStatus(pos, target=10)
+    with mock.patch.object(pos, "stop") as mock_stop:
+        move_st.set_exception(RuntimeError("move error"))
+        mock_stop.assert_called_once()
+        with pytest.raises(RuntimeError):
+            move_st.wait(timeout=10)
+        assert move_st.done is True
+        assert move_st.success is False
